#!/usr/bin/env python3
#-*- coding:utf-8 -*-

# Gery Casiez / Axel Antoine
# 2020

import math
import numpy as np
from scipy import signal
import os.path


def interpolate_at_timestamp(s : [], timestamp: int, d):
    # Get the index of the last event before frametime
    i = 0
    while (i+1 < len(s)) and (s[i+1]["timestamp"] < timestamp) : # <= ?
        i += 1

    if i+1 >= len(s):
        #return s[len(s)-1]['y']
        return float('NaN')

    if s[i+1]['timestamp'] == s[i]['timestamp']:
        return s[i+1]['y']

    alpha = (timestamp-s[i]['timestamp']) / \
            (s[i+1]['timestamp'] - s[i]['timestamp'])
    if d: 
        d['pred-time'] = -(timestamp-s[i]['timestamp'])
    return s[i]['y']+s[i+1]['delta_y']*alpha

def predict_event_at_timestamp(s, event_idx: int, predict_time: int, d, pred=None, predTech = "linear"):
    if event_idx>0 and event_idx<len(s):
        # If prediction time is after last event then predict using 
        # prediction technique
        if predict_time > s[event_idx]["timestamp"]:
            d['pred-time'] = predict_time - s[event_idx]["timestamp"]
            if predTech == "linear":
                return linear_extrapolation(s, event_idx, predict_time)
            else:
                print("predict_event_at_timestamp: prediction technique %s not supported"%predTech)
                sys.exit(-1)
        # prediction time is before last event, we can interpolate
        else:
            return interpolate_at_timestamp(s, predict_time, d)
    return float('NaN')


def linear_extrapolation(s, event_idx: int, predict_time: int):
    velocity = 0
    if s[event_idx]["timestamp"] != s[event_idx-1]["timestamp"]:
        velocity = s[event_idx]['delta_y'] / \
                (s[event_idx]["timestamp"]-s[event_idx-1]["timestamp"])
    return s[event_idx]["y"] + velocity * (predict_time-s[event_idx]["timestamp"])

# Computes jitter metric using Axel formula
# needs output from computeOutputViewPortPos as input
def computeJitterMetricAxelVersion(out: [], pos: str, inter:str) -> float:
    jitters = []
    for i in range(1, len(out)):
        current_diff = out[i][pos] - out[i][inter]
        last_diff = out[i-1][pos] - out[i-1][inter]
        jitters.append(abs(current_diff-last_diff))
    
    return np.nanmean(jitters)


# Remove NAN values from list
def cleanList(l: []) -> []:
    return [x for x in l if not(math.isnan(x))]


def computeGlobalJitter(jitters: []) -> float:
    jitters = cleanList(jitters)

    if len(jitters) > 0:
        return sum(jitters) / len(jitters)
    else:
        # print("Warning from computeGlobalJitter: len(jitters) = 0")
        return float('NaN')

def computeAverageValue(jitters: []) -> float:
    return computeGlobalJitter(jitters)



