#!/usr/bin/env python3
#-*- coding:utf-8 -*-

# Gery Casiez / Axel Antoine
# 2020

import time
import json
import re
import csv
from os import listdir
from os.path import join
from . common import *

def computeOutputViewPortPosFromActualFrameTimes(s: [], delays) -> []:
    """
    To be used to compute the actual jitter from logs.
    Used to compute jitter at 60 Hz with Pixel 3 and 90 Hz with Pixel 4
    """
    out = []

    last_event = None

    i = 1
    last_frame_time = s[1]["frame_time"]
    output_pos = 0

    while i < len(s):
        while (i < len(s)) and (s[i]["frame_time"] == last_frame_time):
            i += 1

        # make sure we handle event only once 
        if last_event != i:
            d = {}
            # frametime and position of the viewport at frametime
            d["output-ts"] = last_frame_time
            d["output-pos"] = s[i-1]['y']

            # Interpolation of the viewport at frametime
            d["input-pos-interpol"] = interpolate_at_timestamp(s, last_frame_time, d)

            # Resampling at frametime with different delays
            for delay in delays:
                predict_time = last_frame_time - delay * 1E3
                d["resampling-delay-%dms"%delay] = \
                    predict_event_at_timestamp(s, i-1, predict_time, d)
                d["input-pos-interpol-%dms"%delay] = interpolate_at_timestamp(s, predict_time, d)

            out.append(d)

        if i < len(s):  
            last_frame_time = s[i]["frame_time"]

        last_event = i


    return out

def processFilesComputeActualOutputJitter(path, delays) -> None:
    start_time = time.time()
    allRes = []
    allStrokes = []

    files = [f for f in listdir(path) if join(path, f).endswith(".json")]

    for f in files:
        print("Processing %s"%f)

        strokes = []
        # load json file and separate strokes
        with open(join(path, f)) as json_file:
            data = json.load(json_file)
            s = []
            pos = 0
            for p in data:
                if p["type"] == "ScrollBegin":
                    if len(s) > 0:
                        strokes.append(s)
                        s = []
                        pos = 0
                pos += p["delta_y"]
                p['y'] = pos
                s.append(p)

        for s in strokes:
            allStrokes.append(s)

        d = {}
        r = re.split('-|_',f)
        d["task"] = r[0]
        d["part"] = r[1]
        d["device"] = r[2]
        if d["device"] == "Pixel3":
            outFreq = 60
        else:
            outFreq = 90
        d["freq"] = outFreq

        outputFreq = float(outFreq)      

        techniques = ["Baseline"]
        for delay in delays:
            techniques.append("Resampling%sms"%delay)
        metrics = ["Axel"]
        jitters = {}

        for s in strokes:
            if len(s) > 4:
                out = computeOutputViewPortPosFromActualFrameTimes(s, delays)

                for t in techniques:
                    for m in metrics:
                        tm = t + "-" + m
                        # Initialize list
                        if tm not in jitters.keys():
                            jitters[tm] = []
                        else:
                            dly = re.findall(r'\d+', t)
                            if len(dly) == 0: # Baseline
                                if m == "Axel":
                                    jitters[tm].append(computeJitterMetricAxelVersion(out, "output-pos","input-pos-interpol"))

                            else:
                                delay = int(dly[0])
                                if m == "Axel":
                                    jitters[tm].append(computeJitterMetricAxelVersion(out, "resampling-delay-%dms"%delay, "input-pos-interpol-%dms"%delay))


        for j in jitters.keys():
            d[j] = computeGlobalJitter(jitters[j])

        allRes.append(d)

    keys = allRes[0].keys()
    with open('dataComparisonPixel3and4.csv', 'w') as output_file:
        dict_writer = csv.DictWriter(output_file, keys)
        dict_writer.writeheader()
        dict_writer.writerows(allRes)

    elapsed_time = time.time() - start_time
    print("Took %s minutes to process."%(elapsed_time / 60.0))

